package be.ASPL.presentationlayer;

import java.io.ByteArrayOutputStream;
import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.nio.ByteBuffer;
import java.util.HashMap;
import java.util.Map.Entry;

import be.SIRAPRISE.util.BracketParser;
import be.SIRAPRISE.util.MyB64;

/**
 * UserSession objects hold all the information regarding a user session.
 * 
 * @author Erwin
 */
class UserSession {

	/**
	 * @param value
	 * @return
	 */
	private static String getUUString (Externalizable value) {
		try {
			final ByteArrayOutputStream b = new ByteArrayOutputStream();
			value.writeExternal(new ObjectOutputStream(b));
			return MyB64.getUUEncodedString(ByteBuffer.wrap(b.toByteArray()));
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * The map that stores the session context data.
	 */
	private HashMap<String, Externalizable> contextData = new HashMap<String, Externalizable>();

	/**
	 * The session's presentation function stack
	 */
	private PresentationFunctionStack functionStack;

	/**
	 * The session ID
	 */
	private long sessionID;

	/**
	 * The session verification token
	 */
	private long sessionVerificationToken;

	/**
	 * The UserData object holding all the information regarding the user that is running this session
	 */
	private UserData userdata;

	/**
	 * Creates a new UserSession object and assigns it the given session ID.
	 * 
	 * @param sessionID
	 *            The ID for this new session
	 */
	UserSession (long sessionID) {
		this.sessionID = sessionID;
		sessionVerificationToken = System.currentTimeMillis() ^ sessionID;
		functionStack = new PresentationFunctionStack();
	}

	/**
	 * @return
	 */
	private String encodeContextData ( ) {
		StringBuilder s = new StringBuilder();
		for (Entry<String, Externalizable> me : contextData.entrySet()) {
			s.append(BracketParser.meta(me.getKey())).append("(CLASSNAME(").append(me.getValue().getClass().getName()).append(")SERIALDATA(").append(getUUString(me.getValue())).append("))"); //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$
		}
		return s.toString();
	}

	/**
	 * @return
	 */
	String encodeState ( ) {
		return "USERDATA(" + userdata.encodeState() + ")FUNCTIONSTACK(" + functionStack.encodeState() + ")SESSIONCONTEXTDATA(" + encodeContextData() + ")"; //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
	}

	/**
	 * Retrieves an object from the user session context
	 * 
	 * @param name
	 *            The name under which something was saved in the context
	 * @return The object saved under the given name
	 */
	Object getFromContext (String name) {
		return contextData.get(name);
	}

	/**
	 * Gets the session's function stack
	 * 
	 * @return the session's function stack
	 */
	PresentationFunctionStack getFunctionStack ( ) {
		return functionStack;
	}

	/**
	 * Gets the session ID
	 * 
	 * @return the session ID
	 */
	long getSessionID ( ) {
		return sessionID;
	}

	/**
	 * Gets the session verification token
	 * 
	 * @return the session verification token
	 */
	long getSessionVerificationToken ( ) {
		return sessionVerificationToken;
	}

	/**
	 * Gets the UserData object
	 * 
	 * @return the UserData object
	 */
	UserData getUserdata ( ) {
		return userdata;
	}

	/**
	 * Saves an object in the user session context
	 * 
	 * @param name
	 *            The name under which something should be saved in the context
	 * @param o
	 *            The object to save under the given name
	 */
	void saveInContext (String name, Externalizable o) {
		contextData.put(name, o);
	}

	/**
	 * Sets the UserData object
	 * 
	 * @param userData
	 *            The UserData object to set
	 */
	void setUserData (UserData userData) {
		this.userdata = userData;
	}
}
